unit uPlugin;
{$INCLUDE 'Plugin.inc'}

interface

implementation

uses
  Windows, SysUtils, Classes,
  {$INCLUDE 'DelphiFarUnits.inc'} ,
  {$IFDEF DEBUG_LOGS} uLog, {$ENDIF}
  uPluginConsts, uMessages,
  uDiscoverDifferences,
  uSynchronizeDirectories,
  uOptions,
  uOptionsUI;

type
  EDirSyncPluginError = class(EFarPluginError);

  TPlugin = class(TFarPlugin)
  private
  protected
    function PluginID: TGUID; override;
    function GetPluginTitle(out Description, Author: string): string; override;
    procedure GetPluginInfo(var Flags: TPluginFlags; var MainMenu, ConfigMenu, DiskMenu: TFarPlugin.TFarPluginMenu; var CommandPrefix: string); override;
    function GetCommonString(ID: TFarPlugin.TCommonString): string; override;
    function Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle; override;
    procedure DoLog(const Msg: string); override;
    function PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean; override;
    function ProcessMacroCall(const Params: array of Variant): boolean;
  public
    constructor Create;
    destructor Destroy; override;
  end;

const
  PLUGIN_TITLE = 'DirSync3';
  PLUGIN_DESCRIPTION = 'Synchronize directories';
  PLUGIN_AUTHOR = 'Pepak, https://www.pepak.net';

{ TPlugin }

constructor TPlugin.Create;
begin
  inherited;
end;

destructor TPlugin.Destroy;
begin
  inherited;
end;

procedure TPlugin.DoLog(const Msg: string);
begin
  {$IFDEF DEBUG_LOGS}
  uLog.Log(Msg);
  {$ENDIF}
end;

function TPlugin.Execute(OpenFrom: DWORD; GUID: PGUID; Data: TIntPtr): THandle;
var
  Diff: TDiscoverDifferences;
  Editor: TSynchronizeTask;
begin
  Result := {$IFDEF FAR3_UP} 0 {$ELSE} INVALID_HANDLE_VALUE {$ENDIF} ;
  if OpenFrom = OPEN_EDITOR then
  begin
    if TSynchronizeTask.FindByEditorID(-1, Editor) then
    begin
      Editor.ExecuteTask;
    end;
  end
  {$IFDEF FAR3_UP}
  else if OpenFrom = OPEN_FROMMACRO_ then
  begin
    if ProcessMacroCall(TFarMacro.MacroParamsToDelphi(Data)) then
      Result := 1;
  end
  {$ENDIF}
  else begin
    Diff := TDiscoverDifferences.Create;
    try
      Result := Diff.Execute;
    finally
      FreeAndNil(Diff);
    end;
  end;
end;

function TPlugin.GetCommonString(ID: TFarPlugin.TCommonString): string;
begin
  case ID of
    TFarPlugin.TCommonString.MTitle:
      Result := GetMsg(MTitle);
    TFarPlugin.TCommonString.MExceptionOccurredIn:
      Result := GetMsg(MExceptionOccurredIn);
    TFarPlugin.TCommonString.MOk:
      Result := GetMsg(MOk);
    TFarPlugin.TCommonString.MCancel:
      Result := GetMsg(MCancel);
    TFarPlugin.TCommonString.MYes:
      Result := GetMsg(MYes);
    TFarPlugin.TCommonString.MNo:
      Result := GetMsg(MNo);
    else
      Result := inherited GetCommonString(ID);
  end;
end;

procedure TPlugin.GetPluginInfo(var Flags: TPluginFlags; var MainMenu, ConfigMenu, DiskMenu: TFarPlugin.TFarPluginMenu; var CommandPrefix: string);
begin
  Flags := Flags + PF_EDITOR;
  SetLength(MainMenu, 1);
  MainMenu[0].Title := GetMsg(MTitle);
  MainMenu[0].ID := MENU_PANEL_GUID;
  SetLength(ConfigMenu, 1);
  ConfigMenu[0].Title := GetMsg(MTitle);
  ConfigMenu[0].ID := MENU_CONFIG_GUID;
end;

function TPlugin.GetPluginTitle(out Description, Author: string): string;
begin
  Result := PLUGIN_TITLE;
  Description := PLUGIN_DESCRIPTION;
  Author := PLUGIN_AUTHOR;
end;

function TPlugin.PluginConfig(GUID: PGUID; MenuItem: TIntPtr): boolean;
var
  Dlg: TOptionsDialog;
begin
  Result := False;
  Dlg := TOptionsDialog.Create;
  try
    Dlg.Options := TOptions.Create;
    try
      Dlg.Options.Load;
      if Dlg.Execute then
        Result := True;
    finally
      Dlg.Options.Free;
      Dlg.Options := nil;
    end;
  finally
    FreeAndNil(Dlg);
  end;
end;

function TPlugin.PluginID: TGUID;
begin
  Result := PLUGIN_GUID;
end;

function TPlugin.ProcessMacroCall(const Params: array of Variant): boolean;

  function GetDirectionFromParameter(ParamIndex: integer): boolean;
    // returns True for Left, False for Right, Exception otherwise
  var
    Value: string;
  begin
    if Length(Params) <= ParamIndex then
      raise EDirSyncPluginError.CreateFmt(GetMsg(MMissingMacroParam), [ParamIndex+1]);
    Value := Params[ParamIndex];
    if Value = 'left' then
      Result := True
    else if Value = 'right' then
      Result := False
    else
      raise EDirSyncPluginError.CreateFmt(GetMsg(MInvalidMacroDirection), [Value]);
  end;

var
  FunctionName: string;
  Editor: TSynchronizeTask;
begin
  Result := False;
  if not TSynchronizeTask.FindByEditorID(-1, Editor) then
    Editor := nil;
  if Length(Params) > 0 then
  begin
    FunctionName := Params[0];
    if FunctionName = 'is_editor' then
    begin
      if Editor <> nil then
        Result := True;
    end
    else if FunctionName = 'is_editor_line' then
    begin
      if Editor <> nil then
        if Editor.IsEditorLine then
          Result := True;
    end
    else if Editor = nil then
    begin
      Result := False;
    end
    else if FunctionName = 'change_direction' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etCopyToLeft)
      else
        Result := Editor.ExecuteTask(etCopyToRight);
    end
    else if FunctionName = 'next_difference' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etFindNextToLeft)
      else
        Result := Editor.ExecuteTask(etFindNextToRight);
    end
    else if FunctionName = 'edit_file' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etEditLeft)
      else
        Result := Editor.ExecuteTask(etEditRight);
    end
    else if FunctionName = 'goto_file' then
    begin
      if GetDirectionFromParameter(1) then
        Result := Editor.ExecuteTask(etGotoLeft)
      else
        Result := Editor.ExecuteTask(etGotoRight);
    end
    else if FunctionName = 'file_info' then
    begin
      Result := Editor.ExecuteTask(etFileInfo);
    end
    else if FunctionName = 'compare' then
    begin
      Result := Editor.ExecuteTask(etCompare);
    end;
  end;
end;

exports
  ProcessEditorEventW,
  ProcessEditorInputW;

initialization
  TFarPlugin.InitInstance(TPlugin.Create);

end.
